@@ -50,7 +50,7 @@ from .addons import AddOnManagerDialog
 from ..document.schemeedit import SchemeEditWidget
 
 from ..scheme import widgetsscheme
-from ..scheme.readwrite import parse_scheme, sniff_version
+from ..scheme.readwrite import scheme_load, sniff_version
 
 from . import welcomedialog
 from ..preview import previewdialog, previewmodel
@@ -398,6 +398,14 @@ class CanvasMainWindow(QMainWindow):
                     icon=canvas_icons("Open.svg")
                     )
 
+        self.open_and_freeze_action = \
+            QAction(self.tr("Open and Freeze"), self,
+                    objectName="action-open-and-freeze",
+                    toolTip=self.tr("Open a new scheme and freeze signal "
+                                    "propagation."),
+                    triggered=self.open_and_freeze_scheme
+                    )
+
         self.save_action = \
             QAction(self.tr("Save"), self,
                     objectName="action-save",
@@ -433,7 +441,7 @@ class CanvasMainWindow(QMainWindow):
         self.get_started_action = \
             QAction(self.tr("Get Started"), self,
                     objectName="get-started-action",
-                    toolTip=self.tr("View a 'Getting Started' video."),
+                    toolTip=self.tr("View a 'Get Started' introduction."),
                     triggered=self.get_started,
                     icon=canvas_icons("Get Started.svg")
                     )
@@ -575,6 +583,7 @@ class CanvasMainWindow(QMainWindow):
         file_menu = QMenu(self.tr("&File"), menu_bar)
         file_menu.addAction(self.new_action)
         file_menu.addAction(self.open_action)
+        file_menu.addAction(self.open_and_freeze_action)
         file_menu.addAction(self.reload_last_action)
 
         # File -> Open Recent submenu
@@ -881,6 +890,24 @@ class CanvasMainWindow(QMainWindow):
         else:
             return QDialog.Rejected
 
+    def open_and_freeze_scheme(self):
+        """
+        Open a new scheme and freeze signal propagation. Return
+        QDialog.Rejected if the user canceled the operation and
+        QDialog.Accepted otherwise.
+
+        """
+        frozen = self.freeze_action.isChecked()
+        if not frozen:
+            self.freeze_action.trigger()
+
+        state = self.open_scheme()
+        if state == QDialog.Rejected:
+            # If the action was rejected restore the original frozen state
+            if not frozen:
+                self.freeze_action.trigger()
+        return state
+
     def open_scheme_file(self, filename):
         """
         Open and load a scheme file.
@@ -924,9 +951,9 @@ class CanvasMainWindow(QMainWindow):
         new_scheme = widgetsscheme.WidgetsScheme(parent=self)
         errors = []
         try:
-            parse_scheme(new_scheme, open(filename, "rb"),
-                         error_handler=errors.append,
-                         allow_pickle_data=True)
+            scheme_load(new_scheme, open(filename, "rb"),
+                        error_handler=errors.append)
+
         except Exception:
             message_critical(
                  self.tr("Could not load an Orange Scheme file"),
