@@ -1,7 +1,7 @@
 """
 <name>File</name>
 <description>Reads data from a file.</description>
-<icon>icons/File.png</icon>
+<icon>icons/File.svg</icon>
 <contact>Janez Demsar (janez.demsar(@at@)fri.uni-lj.si)</contact>
 <priority>10</priority>
 """
@@ -14,7 +14,42 @@ from OWWidget import *
 import OWGUI, string, os, sys, warnings
 import orngIO
 
-warnings.filterwarnings("error", ".*" , orange.KernelWarning, "OWFile", 11)
+NAME = "File"
+ID = "orange.widgets.data.file"
+
+DESCRIPTION = """
+Read a data table from a supported file format on the the file system and
+send it to the the output.
+"""
+
+LONG_DESCRIPTION = """
+This is the widget you will probably use at the start of every schema to read
+the input data file (data table with examples). The widget maintains a
+history of most recently used data files. For convenience, the history
+also includes a directory with the sample data sets that come with Orange.
+"""
+
+ICON = "icons/File.svg"
+AUTHOR = "Janez Demsar"
+MAINTAINER_EMAIL = "janez.demsar(@at@)fri.uni-lj.si"
+PRIORITY = 10
+CATEGORY = "Data"
+
+KEYWORDS = ["data", "file", "load", "read"]
+
+OUTPUTS = (
+    {"name": "Data",
+     "type": orange.ExampleTable,
+     "doc": "Attribute-valued data set read from the input file.",
+    },
+)
+
+WIDGET_CLASS = "OWFile"
+
+# This is why the 'call''s line number is important. Actually you can
+# move it but you need to make sure the following filter's lineno is updated
+warnings.filterwarnings("error", ".*", orange.KernelWarning, "OWFile", 11)
+
 
 class FileNameContextHandler(ContextHandler):
     def match(self, context, imperfect, filename):
@@ -28,7 +63,7 @@ def addOrigin(examples, filename):
     for var in strings:
         if "type" in var.attributes and "origin" not in var.attributes:
             var.attributes["origin"] = dirname
-                 
+
 
 class OWFile(OWWidget):
     settingsList=["recentFiles", "createNewOn", "showAdvanced"]
@@ -44,7 +79,6 @@ class OWFile(OWWidget):
                ".basket": "Basket file"}
     formats.update(dict((ft[1][2:], ft[0]) for ft in registeredFileTypes))
      
-                 
     def __init__(self, parent=None, signalManager = None):
         OWWidget.__init__(self, parent, signalManager, "File", wantMainArea = 0, resizingEnabled = 1)
 
