@@ -1,18 +1,34 @@
 """
 <name>Python Script</name>
 <description>Executes python script.</description>
-<icon>icons/PythonScript.png</icon>
-<contact>Miha Stajdohar (miha.stajdohar(@at@)gmail.com)</contact> 
+<icon>icons/PythonScript.svg</icon>
+<contact>Miha Stajdohar (miha.stajdohar(@at@)gmail.com)</contact>
 <priority>3150</priority>
 """
+import sys
+import os
+import code
+import keyword
+import itertools
+
+from PyQt4.QtGui import (
+    QSyntaxHighlighter, QPlainTextEdit, QTextCharFormat, QTextCursor,
+    QTextDocument, QPlainTextDocumentLayout, QBrush, QFont, QColor, QPalette,
+    QStyledItemDelegate, QStyleOptionViewItemV4, QLineEdit, QListView,
+    QSizePolicy, QAction, QMenu, QKeySequence, QSplitter, QToolButton,
+    QItemSelectionModel, QFileDialog
+)
+
+from PyQt4.QtCore import Qt, QRegExp, QByteArray
+from PyQt4.QtCore import SIGNAL
+
 from OWWidget import *
 
-import sys, traceback
+from OWItemModels import PyListModel, ModelActionsWidget
+
 import OWGUI
 import Orange
 
-import code
-
 
 class PythonSyntaxHighlighter(QSyntaxHighlighter):
     def __init__(self, parent=None):
@@ -28,21 +44,26 @@ class PythonSyntaxHighlighter(QSyntaxHighlighter):
         self.commentFormat.setForeground(QBrush(Qt.lightGray))
         self.decoratorFormat = QTextCharFormat()
         self.decoratorFormat.setForeground(QBrush(Qt.darkGray))
-        
-        self.keywords = ["def", "if", "else", "elif", "for", "while", "with", "try", "except",
-                         "finally", "not", "in", "lambda", "None", "import", "class", "return", "print",
-                         "yield", "break", "continue", "raise", "or", "and", "True", "False", "pass",
-                         "from", "as"]
-        self.rules = [(QRegExp(r"\b%s\b" % keyword), self.keywordFormat) for keyword in self.keywords] + \
-                     [(QRegExp(r"\bdef\s+([A-Za-z_]+[A-Za-z0-9_]+)\s*\("), self.defFormat),
-                      (QRegExp(r"\bclass\s+([A-Za-z_]+[A-Za-z0-9_]+)\s*\("), self.defFormat),
+
+        self.keywords = list(keyword.kwlist)
+
+        self.rules = [(QRegExp(r"\b%s\b" % kwd), self.keywordFormat)
+                      for kwd in self.keywords] + \
+                     [(QRegExp(r"\bdef\s+([A-Za-z_]+[A-Za-z0-9_]+)\s*\("),
+                       self.defFormat),
+                      (QRegExp(r"\bclass\s+([A-Za-z_]+[A-Za-z0-9_]+)\s*\("),
+                       self.defFormat),
                       (QRegExp(r"'.*'"), self.stringFormat),
                       (QRegExp(r'".*"'), self.stringFormat),
                       (QRegExp(r"#.*"), self.commentFormat),
-                      (QRegExp(r"@[A-Za-z_]+[A-Za-z0-9_]+"), self.decoratorFormat)]
-                     
+                      (QRegExp(r"@[A-Za-z_]+[A-Za-z0-9_]+"),
+                       self.decoratorFormat)]
+
+        self.multilineStart = QRegExp(r"(''')|" + r'(""")')
+        self.multilineEnd = QRegExp(r"(''')|" + r'(""")')
+
         QSyntaxHighlighter.__init__(self, parent)
-        
+
     def highlightBlock(self, text):
         for pattern, format in self.rules:
             exp = QRegExp(pattern)
@@ -54,10 +75,11 @@ class PythonSyntaxHighlighter(QSyntaxHighlighter):
                 else:
                     self.setFormat(exp.pos(0), len(str(exp.cap(0))), format)
                 index = exp.indexIn(text, index + length)
-                
-        ## Multi line strings
-        start = QRegExp(r"(''')|" + r'(""")')
-        end = QRegExp(r"(''')|" + r'(""")')
+
+        # Multi line strings
+        start = self.multilineStart
+        end = self.multilineEnd
+
         self.setCurrentBlockState(0)
         startIndex, skip = 0, 0
         if self.previousBlockState() != 1:
@@ -70,22 +92,26 @@ class PythonSyntaxHighlighter(QSyntaxHighlighter):
             else:
                 commentLen = endIndex - startIndex + 3
             self.setFormat(startIndex, commentLen, self.stringFormat)
-            startIndex, skip = start.indexIn(text, startIndex + commentLen + 3), 3
-                
+            startIndex, skip = (start.indexIn(text,
+                                              startIndex + commentLen + 3),
+                                3)
+
+
 class PythonScriptEditor(QPlainTextEdit):
     INDENT = 4
+
     def lastLine(self):
         text = str(self.toPlainText())
         pos = self.textCursor().position()
         index = text.rfind("\n", 0, pos)
         text = text[index: pos].lstrip("\n")
         return text
-    
+
     def keyPressEvent(self, event):
         if event.key() == Qt.Key_Return:
             text = self.lastLine()
             indent = len(text) - len(text.lstrip())
-            if text.strip() == "pass" or text.strip().startswith("return"):
+            if text.strip() == "pass" or text.strip().startswith("return "):
                 indent = max(0, indent - self.INDENT)
             elif text.strip().endswith(":"):
                 indent += self.INDENT
@@ -101,21 +127,22 @@ class PythonScriptEditor(QPlainTextEdit):
                     cursor.deletePreviousChar()
             else:
                 QPlainTextEdit.keyPressEvent(self, event)
-                
+
         else:
             QPlainTextEdit.keyPressEvent(self, event)
-        
+
+
 class PythonConsole(QPlainTextEdit, code.InteractiveConsole):
-    def __init__(self, locals = None, parent=None):
+    def __init__(self, locals=None, parent=None):
         QPlainTextEdit.__init__(self, parent)
         code.InteractiveConsole.__init__(self, locals)
         self.history, self.historyInd = [""], 0
         self.loop = self.interact()
         self.loop.next()
-        
+
     def setLocals(self, locals):
         self.locals = locals
-        
+
     def interact(self, banner=None):
         try:
             sys.ps1
@@ -125,7 +152,8 @@ class PythonConsole(QPlainTextEdit, code.InteractiveConsole):
             sys.ps2
         except AttributeError:
             sys.ps2 = "... "
-        cprt = 'Type "help", "copyright", "credits" or "license" for more information.'
+        cprt = ('Type "help", "copyright", "credits" or "license" '
+                'for more information.')
         if banner is None:
             self.write("Python %s on %s\n%s\n(%s)\n" %
                        (sys.version, sys.platform, cprt,
@@ -152,34 +180,34 @@ class PythonConsole(QPlainTextEdit, code.InteractiveConsole):
                 self.write("\nKeyboardInterrupt\n")
                 self.resetbuffer()
                 more = 0
-                
+
     def raw_input(self, prompt):
         input = str(self.document().lastBlock().previous().text())
         return input[len(prompt):]
-        
+
     def new_prompt(self, prompt):
         self.write(prompt)
         self.newPromptPos = self.textCursor().position()
-        
+
     def write(self, data):
         cursor = QTextCursor(self.document())
         cursor.movePosition(QTextCursor.End, QTextCursor.MoveAnchor)
         cursor.insertText(data)
         self.setTextCursor(cursor)
         self.ensureCursorVisible()
-        
+
     def push(self, line):
         if self.history[0] != line:
             self.history.insert(0, line)
         self.historyInd = 0
-        more = 0
+
         saved = sys.stdout, sys.stderr
         try:
             sys.stdout, sys.stderr = self, self
             return code.InteractiveConsole.push(self, line)
         finally:
             sys.stdout, sys.stderr = saved
-            
+
     def setLine(self, line):
         cursor = QTextCursor(self.document())
         cursor.movePosition(QTextCursor.End)
@@ -187,7 +215,7 @@ class PythonConsole(QPlainTextEdit, code.InteractiveConsole):
         cursor.removeSelectedText()
         cursor.insertText(line)
         self.setTextCursor(cursor)
-        
+
     def keyPressEvent(self, event):
         if event.key() == Qt.Key_Return:
             self.write("\n")
@@ -203,23 +231,76 @@ class PythonConsole(QPlainTextEdit, code.InteractiveConsole):
                 QPlainTextEdit.keyPressEvent(self, event)
         else:
             QPlainTextEdit.keyPressEvent(self, event)
-            
+
     def historyUp(self):
         self.setLine(self.history[self.historyInd])
         self.historyInd = min(self.historyInd + 1, len(self.history) - 1)
-        
+
     def historyDown(self):
         self.setLine(self.history[self.historyInd])
         self.historyInd = max(self.historyInd - 1, 0)
-        
+
     def complete(self):
         pass
-    
-from OWItemModels import PyListModel, ModelActionsWidget
+
+    def _moveCursorToInputLine(self):
+        """
+        Move the cursor to the input line if not already there. If the cursor
+        if already in the input line (at position greater or equal to
+        `newPromptPos`) it is left unchanged, otherwise it is moved at the
+        end.
+
+        """
+        cursor = self.textCursor()
+        pos = cursor.position()
+        if pos < self.newPromptPos:
+            cursor.movePosition(QTextCursor.End)
+            self.setTextCursor(cursor)
+
+    def pasteCode(self, source):
+        """
+        Paste source code into the console.
+        """
+        self._moveCursorToInputLine()
+
+        for line in interleave(source.splitlines(), itertools.repeat("\n")):
+            if line != "\n":
+                self.insertPlainText(line)
+            else:
+                self.write("\n")
+                self.loop.next()
+
+    def insertFromMimeData(self, source):
+        """
+        Reimplemented from QPlainTextEdit.insertFromMimeData.
+        """
+        if source.hasText():
+            self.pasteCode(unicode(source.text()))
+            return
+
+
+def interleave(seq1, seq2):
+    """
+    Interleave elements of `seq2` between consecutive elements of `seq1`.
+
+        >>> list(interleave([1, 3, 5], [2, 4]))
+        [1, 2, 3, 4, 5]
+
+    """
+    iterator1, iterator2 = iter(seq1), iter(seq2)
+    leading = next(iterator1)
+    for element in iterator1:
+        yield leading
+        yield next(iterator2)
+        leading = element
+
+    yield leading
+
 
 class Script(object):
     Modified = 1
-    MissingFromFilesystem = 2 
+    MissingFromFilesystem = 2
+
     def __init__(self, name, script, flags=0, sourceFileName=None):
         self.name = name
         self.script = script
@@ -227,163 +308,182 @@ class Script(object):
         self.sourceFileName = sourceFileName
         self.modifiedScript = None
 
+
 class ScriptItemDelegate(QStyledItemDelegate):
     def __init__(self, parent):
         QStyledItemDelegate.__init__(self, parent)
-        
+
     def displayText(self, variant, locale):
         script = variant.toPyObject()
         if script.flags & Script.Modified:
             return QString("*" + script.name)
         else:
             return QString(script.name)
-    
+
     def paint(self, painter, option, index):
         script = index.data(Qt.DisplayRole).toPyObject()
-        tmp_palette = None
+
         if script.flags & Script.Modified:
             option = QStyleOptionViewItemV4(option)
             option.palette.setColor(QPalette.Text, QColor(Qt.red))
             option.palette.setColor(QPalette.Highlight, QColor(Qt.darkRed))
         QStyledItemDelegate.paint(self, painter, option, index)
 
-        
     def createEditor(self, parent, option, index):
         return QLineEdit(parent)
-    
+
     def setEditorData(self, editor, index):
         script = index.data(Qt.DisplayRole).toPyObject()
         editor.setText(script.name)
-        
+
     def setModelData(self, editor, model, index):
         model[index.row()].name = str(editor.text())
-        
+
+
 class OWPythonScript(OWWidget):
-    
-    settingsList = ["codeFile", "libraryListSource", "currentScriptIndex", "splitterState"]
-                    
+
+    settingsList = ["codeFile", "libraryListSource", "currentScriptIndex",
+                    "splitterState", "auto_execute"]
+
     def __init__(self, parent=None, signalManager=None):
         OWWidget.__init__(self, parent, signalManager, 'Python Script')
-        
-        self.inputs = [("in_data", ExampleTable, self.setExampleTable), ("in_distance", orange.SymMatrix, self.setDistanceMatrix), ("in_learner", orange.Learner, self.setLearner), ("in_classifier", orange.Classifier, self.setClassifier)]
-        self.outputs = [("out_data", ExampleTable), ("out_distance", orange.SymMatrix), ("out_learner", orange.Learner), ("out_classifier", orange.Classifier, Dynamic)]
 
-        try:
-            self.inputs.append(("in_network", Orange.network.Graph, self.setNetwork))
-            self.outputs.append(("out_network", Orange.network.Graph))
-        except:
-            pass
+        self.inputs = [("in_data", Orange.data.Table, self.setExampleTable,
+                        Default),
+                       ("in_distance", Orange.misc.SymMatrix,
+                        self.setDistanceMatrix, Default),
+                       ("in_learner", Orange.core.Learner, self.setLearner,
+                        Default),
+                       ("in_classifier", Orange.core.Classifier,
+                        self.setClassifier, Default),
+                       ("in_object", object, self.setObject)]
+
+        self.outputs = [("out_data", Orange.data.Table),
+                        ("out_distance", Orange.misc.SymMatrix),
+                        ("out_learner", Orange.core.Learner),
+                        ("out_classifier", Orange.core.Classifier, Dynamic),
+                        ("out_object", object, Dynamic)]
 
         self.in_data = None
-        self.in_network = None
         self.in_distance = None
         self.in_learner = None
         self.in_classifier = None
-        
+        self.in_object = None
+        self.auto_execute = False
+
         self.codeFile = ''
-        self.libraryListSource = [Script("Hello world", "print 'Hello world'\n")]
+        self.libraryListSource = [Script("Hello world",
+                                         "print 'Hello world'\n")]
         self.currentScriptIndex = 0
         self.splitterState = None
         self.loadSettings()
-        
+
         for s in self.libraryListSource:
             s.flags = 0
-        
+
         self._cachedDocuments = {}
-        
+
         self.infoBox = OWGUI.widgetBox(self.controlArea, 'Info')
-        label = OWGUI.label(self.infoBox, self, "<p>Execute python script.</p><p>Input variables:<ul><li> " + \
-                    "<li>".join(t[0] for t in self.inputs) + "</ul></p><p>Output variables:<ul><li>" + \
-                    "<li>".join(t[0] for t in self.outputs) + "</ul></p>")
-        self.libraryList = PyListModel([], self, flags=Qt.ItemIsSelectable | Qt.ItemIsEnabled | Qt.ItemIsEditable)
-#        self.libraryList.append(Script("Hello world", "print 'Hello world'\n"))
+        OWGUI.label(
+            self.infoBox, self,
+            "<p>Execute python script.</p><p>Input variables:<ul><li> " + \
+            "<li>".join(t[0] for t in self.inputs) + \
+            "</ul></p><p>Output variables:<ul><li>" + \
+            "<li>".join(t[0] for t in self.outputs) + \
+            "</ul></p>"
+        )
+
+        self.libraryList = PyListModel([], self, flags=Qt.ItemIsSelectable | \
+                                       Qt.ItemIsEnabled | Qt.ItemIsEditable)
+
         self.libraryList.wrap(self.libraryListSource)
-        
+
         self.controlBox = OWGUI.widgetBox(self.controlArea, 'Library')
         self.controlBox.layout().setSpacing(1)
+
         self.libraryView = QListView()
-        self.libraryView.setEditTriggers(QListView.DoubleClicked | QListView.EditKeyPressed)
-        self.libraryView.setSizePolicy(QSizePolicy.Ignored, QSizePolicy.Preferred)
+        self.libraryView.setEditTriggers(QListView.DoubleClicked | \
+                                         QListView.EditKeyPressed)
+        self.libraryView.setSizePolicy(QSizePolicy.Ignored,
+                                       QSizePolicy.Preferred)
         self.libraryView.setItemDelegate(ScriptItemDelegate(self))
         self.libraryView.setModel(self.libraryList)
-        self.connect(self.libraryView.selectionModel(), SIGNAL("selectionChanged(QItemSelection, QItemSelection)"), self.onSelectedScriptChanged)
+
+        self.connect(self.libraryView.selectionModel(),
+                     SIGNAL("selectionChanged(QItemSelection, QItemSelection)"),
+                     self.onSelectedScriptChanged)
         self.controlBox.layout().addWidget(self.libraryView)
+
         w = ModelActionsWidget()
-        
+
         self.addNewScriptAction = action = QAction("+", self)
-        action.pyqtConfigure(toolTip="Add a new script to the library")
+        action.setToolTip("Add a new script to the library")
         self.connect(action, SIGNAL("triggered()"), self.onAddScript)
-        new_empty = QAction("Add a new empty script", action)
-        new_from_file = QAction("Add a new script from a file", action)
-        self.connect(new_empty, SIGNAL("triggered()"), self.onAddScript)
-        self.connect(new_from_file, SIGNAL("triggered()"), self.onAddScriptFromFile)
-        menu = QMenu(w)
-        menu.addAction(new_empty)
-        menu.addAction(new_from_file)
-        
-#        action.setMenu(menu)
-        button = w.addAction(action)
-        
+        w.addAction(action)
+
         self.removeAction = action = QAction("-", self)
-        action.pyqtConfigure(toolTip="Remove script from library")
+        action.setToolTip("Remove script from library")
         self.connect(action, SIGNAL("triggered()"), self.onRemoveScript)
         w.addAction(action)
-        
+
         action = QAction("Update", self)
-        action.pyqtConfigure(toolTip="Save changes in the editor to library")
+        action.setToolTip("Save changes in the editor to library")
         action.setShortcut(QKeySequence(QKeySequence.Save))
-        self.connect(action, SIGNAL("triggered()"), self.commitChangesToLibrary)
-        b = w.addAction(action)
-#        b.setSizePolicy(QSizePolicy.MinimumExpanding, QSizePolicy.Fixed)
-        
+        self.connect(action, SIGNAL("triggered()"),
+                     self.commitChangesToLibrary)
+        w.addAction(action)
+
         action = QAction("More", self)
-        action.pyqtConfigure(toolTip="More actions") #, icon=self.style().standardIcon(QStyle.SP_ToolBarHorizontalExtensionButton))
-        
-        self.openScriptFromFileAction = new_from_file = QAction("Import a script from a file", self)
-        self.saveScriptToFile = save_to_file = QAction("Save selected script to a file", self)
+        action.pyqtConfigure(toolTip="More actions")
+
+        new_from_file = QAction("Import a script from a file", self)
+        save_to_file = QAction("Save selected script to a file", self)
         save_to_file.setShortcut(QKeySequence(QKeySequence.SaveAs))
-        
-        self.connect(new_from_file, SIGNAL("triggered()"), self.onAddScriptFromFile)
+
+        self.connect(new_from_file, SIGNAL("triggered()"),
+                     self.onAddScriptFromFile)
         self.connect(save_to_file, SIGNAL("triggered()"), self.saveScript)
-        
+
         menu = QMenu(w)
         menu.addAction(new_from_file)
         menu.addAction(save_to_file)
         action.setMenu(menu)
-        b = w.addAction(action)
-        b.setPopupMode(QToolButton.InstantPopup) 
-        ## TODO: set the space for the indicator
-        
+        button = w.addAction(action)
+        button.setPopupMode(QToolButton.InstantPopup)
+
         w.layout().setSpacing(1)
-        
+
         self.controlBox.layout().addWidget(w)
-                    
-#        OWGUI.button(self.controlBox, self, "Open...", callback=self.openScript)
-#        OWGUI.button(self.controlBox, self, "Save...", callback=self.saveScript)
-        
+
         self.runBox = OWGUI.widgetBox(self.controlArea, 'Run')
         OWGUI.button(self.runBox, self, "Execute", callback=self.execute)
-        
+        OWGUI.checkBox(self.runBox, self, "auto_execute", "Auto execute",
+                       tooltip=("Run the script automatically whenever "
+                                "the inputs to the widget change."))
+
         self.splitCanvas = QSplitter(Qt.Vertical, self.mainArea)
         self.mainArea.layout().addWidget(self.splitCanvas)
-        
-        self.defaultFont = defaultFont = "Monaco" if sys.platform == "darwin" else "Courier"
+
+        self.defaultFont = defaultFont = \
+            "Monaco" if sys.platform == "darwin" else "Courier"
+
         self.textBox = OWGUI.widgetBox(self, 'Python script')
         self.splitCanvas.addWidget(self.textBox)
         self.text = PythonScriptEditor(self)
         self.textBox.layout().addWidget(self.text)
-        
+
         self.textBox.setAlignment(Qt.AlignVCenter)
         self.text.setTabStopWidth(4)
-        
-        self.connect(self.text, SIGNAL("modificationChanged(bool)"), self.onModificationChanged)
-        
+
+        self.connect(self.text, SIGNAL("modificationChanged(bool)"),
+                     self.onModificationChanged)
+
         self.saveAction = action = QAction("&Save", self.text)
-        action.pyqtConfigure(toolTip="Save script to file")
+        action.setToolTip("Save script to file")
         action.setShortcut(QKeySequence(QKeySequence.Save))
         action.setShortcutContext(Qt.WidgetWithChildrenShortcut)
         self.connect(action, SIGNAL("triggered()"), self.saveScript)
-        
+
         self.consoleBox = OWGUI.widgetBox(self, 'Console')
         self.splitCanvas.addWidget(self.consoleBox)
         self.console = PythonConsole(self.__dict__, self)
@@ -391,133 +491,166 @@ class OWPythonScript(OWWidget):
         self.console.document().setDefaultFont(QFont(defaultFont))
         self.consoleBox.setAlignment(Qt.AlignBottom)
         self.console.setTabStopWidth(4)
-        
+
         self.openScript(self.codeFile)
         try:
-            self.libraryView.selectionModel().select(self.libraryList.index(self.currentScriptIndex), QItemSelectionModel.ClearAndSelect)
+            self.libraryView.selectionModel().select(
+                self.libraryList.index(self.currentScriptIndex),
+                QItemSelectionModel.ClearAndSelect
+            )
         except Exception:
             pass
+
         self.splitCanvas.setSizes([2, 1])
         if self.splitterState is not None:
             self.splitCanvas.restoreState(QByteArray(self.splitterState))
-        
-        self.connect(self.splitCanvas, SIGNAL("splitterMoved(int, int)"), lambda pos, ind: setattr(self, "splitterState", str(self.splitCanvas.saveState())))
+
+        self.connect(self.splitCanvas, SIGNAL("splitterMoved(int, int)"),
+                     self.onSpliterMoved)
         self.controlArea.layout().addStretch(1)
-        self.resize(800,600)
-        
+        self.resize(800, 600)
+
     def setExampleTable(self, et):
         self.in_data = et
-        
+
     def setDistanceMatrix(self, dm):
         self.in_distance = dm
-        
-    def setNetwork(self, net):
-        self.in_network = net
-        
+
     def setLearner(self, learner):
         self.in_learner = learner
-        
+
     def setClassifier(self, classifier):
         self.in_classifier = classifier
-        
+
+    def setObject(self, obj):
+        self.in_object = obj
+
+    def handleNewSignals(self):
+        if self.auto_execute:
+            self.execute()
+
     def selectedScriptIndex(self):
         rows = self.libraryView.selectionModel().selectedRows()
         if rows:
             return  [i.row() for i in rows][0]
         else:
             return None
-        
+
     def setSelectedScript(self, index):
         selection = self.libraryView.selectionModel()
-        selection.select(self.libraryList.index(index), QItemSelectionModel.ClearAndSelect)
-        
+        selection.select(self.libraryList.index(index),
+                         QItemSelectionModel.ClearAndSelect)
+
     def onAddScript(self, *args):
         self.libraryList.append(Script("New script", "", 0))
         self.setSelectedScript(len(self.libraryList) - 1)
-        
+
     def onAddScriptFromFile(self, *args):
-        file = QFileDialog.getOpenFileName(self, 'Open Python Script', self.codeFile, 'Python files (*.py)\nAll files(*.*)')
-        file = unicode(file)
-        if file:
-            name = os.path.basename(file)
-            self.libraryList.append(Script(name, open(file, "rb").read(), 0, file))
+        filename = QFileDialog.getOpenFileName(
+            self, 'Open Python Script',
+            self.codeFile,
+            'Python files (*.py)\nAll files(*.*)'
+        )
+
+        filename = unicode(filename)
+        if filename:
+            name = os.path.basename(filename)
+            self.libraryList.append(Script(name, open(filename, "rb").read(),
+                                           0, filename))
             self.setSelectedScript(len(self.libraryList) - 1)
-    
+
     def onRemoveScript(self, *args):
         index = self.selectedScriptIndex()
         if index is not None:
             del self.libraryList[index]
-            self.libraryView.selectionModel().select(self.libraryList.index(max(index - 1, 0)), QItemSelectionModel.ClearAndSelect)
-    
+
+            self.libraryView.selectionModel().select(
+                self.libraryList.index(max(index - 1, 0)),
+                QItemSelectionModel.ClearAndSelect
+            )
+
     def onSaveScriptToFile(self, *args):
         index = self.selectedScriptIndex()
         if index is not None:
             self.saveScript()
-            
+
     def onSelectedScriptChanged(self, selected, deselected):
         index = [i.row() for i in selected.indexes()]
         if index:
-            current = index[0] 
+            current = index[0]
             if current >= len(self.libraryList):
                 self.addNewScriptAction.trigger()
                 return
+
             self.text.setDocument(self.documentForScript(current))
             self.currentScriptIndex = current
-            
+
     def documentForScript(self, script=0):
         if type(script) != Script:
             script = self.libraryList[script]
+
         if script not in self._cachedDocuments:
             doc = QTextDocument(self)
             doc.setDocumentLayout(QPlainTextDocumentLayout(doc))
             doc.setPlainText(script.script)
             doc.setDefaultFont(QFont(self.defaultFont))
             doc.highlighter = PythonSyntaxHighlighter(doc)
-            self.connect(doc, SIGNAL("modificationChanged(bool)"), self.onModificationChanged)
+            self.connect(doc, SIGNAL("modificationChanged(bool)"),
+                         self.onModificationChanged)
             doc.setModified(False)
             self._cachedDocuments[script] = doc
         return self._cachedDocuments[script]
-    
+
     def commitChangesToLibrary(self, *args):
         index = self.selectedScriptIndex()
         if index is not None:
             self.libraryList[index].script = self.text.toPlainText()
             self.text.document().setModified(False)
             self.libraryList.emitDataChanged(index)
-            
+
     def onModificationChanged(self, modified):
         index = self.selectedScriptIndex()
         if index is not None:
             self.libraryList[index].flags = Script.Modified if modified else 0
             self.libraryList.emitDataChanged(index)
-        
+
+    def onSpliterMoved(self, pos, ind):
+        self.splitterState = str(self.splitCanvas.saveState())
+
     def updateSelecetdScriptState(self):
         index = self.selectedScriptIndex()
         if index is not None:
             script = self.libraryList[index]
-            self.libraryList[index] = Script(script.name, self.text.toPlainText(), 0)
-            
+            self.libraryList[index] = Script(script.name,
+                                             self.text.toPlainText(),
+                                             0)
+
     def openScript(self, filename=None):
         if filename == None:
-            filename = unicode(QFileDialog.getOpenFileName(self, 'Open Python Script',
-                               self.codeFile, 'Python files (*.py)\nAll files(*.*)'))
+            filename = QFileDialog.getOpenFileName(
+                self, 'Open Python Script',
+                self.codeFile,
+                'Python files (*.py)\nAll files(*.*)'
+            )
+
+            filename = unicode(filename)
             self.codeFile = filename
         else:
             self.codeFile = filename
-            
+
         if self.codeFile == "":
             return
-            
+
         self.error(0)
         try:
             f = open(self.codeFile, 'r')
         except (IOError, OSError), ex:
             self.text.setPlainText("")
             return
-        
+
         self.text.setPlainText(f.read())
         f.close()
-    
+
     def saveScript(self):
         index = self.selectedScriptIndex()
         if index is not None:
@@ -525,10 +658,14 @@ class OWPythonScript(OWWidget):
             filename = script.sourceFileName or self.codeFile
         else:
             filename = self.codeFile
-        filename = QFileDialog.getSaveFileName(self, 'Save Python Script',
-                            filename, 'Python files (*.py)\nAll files(*.*)') 
+        filename = QFileDialog.getSaveFileName(
+            self, 'Save Python Script',
+            filename,
+            'Python files (*.py)\nAll files(*.*)'
+        )
+
         self.codeFile = unicode(filename)
-        
+
         if self.codeFile:
             fn = ""
             head, tail = os.path.splitext(self.codeFile)
@@ -536,11 +673,11 @@ class OWPythonScript(OWWidget):
                 fn = head + ".py"
             else:
                 fn = self.codeFile
-            
+
             f = open(fn, 'w')
             f.write(self.text.toPlainText())
             f.close()
-            
+
     def execute(self):
         self._script = str(self.text.toPlainText())
         self.console.write("\nRunning script:\n")
@@ -550,7 +687,8 @@ class OWPythonScript(OWWidget):
             signal = out[0]
             self.send(signal, getattr(self, signal, None))
 
-if __name__=="__main__":    
+
+if __name__ == "__main__":
     appl = QApplication(sys.argv)
     ow = OWPythonScript()
     ow.show()
