@@ -2,7 +2,7 @@
 <name>Earth</name>
 <description>Multivariate Adaptive Regression Splines (MARS)</description>
 <category>Regression</category>
-<icon>icons/Earth.png<icon>
+<icon>icons/EarthMars.svg</icon>
 <priority>100</priority>
 <tags>MARS, Multivariate, Adaptive, Regression, Splines</tags>
 """
@@ -13,16 +13,19 @@ import Orange
 
 from Orange.regression import earth
 from orngWrap import PreprocessedLearner
- 
+
+
 class OWEarth(OWWidget):
     settingsList = ["name", "degree", "terms", "penalty"]
 
     def __init__(self, parent=None, signalManager=None,
                  title="Earth"):
-        OWWidget.__init__(self, parent, signalManager, title, wantMainArea=False)
+        OWWidget.__init__(self, parent, signalManager, title,
+                          wantMainArea=False)
 
         self.inputs = [("Data", Orange.data.Table, self.set_data),
-                       ("Preprocessor", PreprocessedLearner, self.set_preprocessor)]
+                       ("Preprocessor", PreprocessedLearner,
+                        self.set_preprocessor)]
 
         self.outputs = [("Learner", earth.EarthLearner, Default),
                         ("Predictor", earth.EarthClassifier, Default),
@@ -39,17 +42,19 @@ class OWEarth(OWWidget):
         # GUI
         #####
 
-        OWGUI.lineEdit(self.controlArea, self, "name", 
+        OWGUI.lineEdit(self.controlArea, self, "name",
                        box="Learner/Classifier Name",
                        tooltip="Name for the learner/predictor")
 
         box = OWGUI.widgetBox(self.controlArea, "Forward Pass", addSpace=True)
         OWGUI.spin(box, self, "degree", 1, 3, step=1,
-                   label="Max. term degree", 
-                   tooltip="Maximum degree of the terms derived (number of hinge functions).")
+                   label="Max. term degree",
+                   tooltip="Maximum degree of the terms derived "
+                           "(number of hinge functions).")
         s = OWGUI.spin(box, self, "terms", 1, 200, step=1,
                        label="Max. terms",
-                       tooltip="Maximum number of terms derived in the forward pass.")
+                       tooltip="Maximum number of terms derived in the "
+                               "forward pass.")
         s.control.setSpecialValueText("Automatic")
 
         box = OWGUI.widgetBox(self.controlArea, "Pruning Pass", addSpace=True)
@@ -75,10 +80,12 @@ class OWEarth(OWWidget):
         self.apply()
 
     def apply(self):
-        learner = earth.EarthLearner(degree=self.degree,
-                                    terms=self.terms if self.terms >= 2 else None,
-                                    penalty=self.penalty,
-                                    name=self.name)
+        learner = earth.EarthLearner(
+            degree=self.degree,
+            terms=self.terms if self.terms >= 2 else None,
+            penalty=self.penalty,
+            name=self.name)
+
         predictor = None
         basis_matrix = None
         if self.preprocessor:
@@ -92,10 +99,13 @@ class OWEarth(OWWidget):
             except Exception, ex:
                 self.error(0, "An error during learning: %r" % ex)
 
-            if predictor is not None:     
+            if predictor is not None:
                 base_features = predictor.base_features()
-                basis_domain = Orange.data.Domain(base_features, 
-                                                  self.data.domain.class_var)
+                basis_domain = Orange.data.Domain(
+                    base_features,
+                    self.data.domain.class_var,
+                    self.data.domain.class_vars)
+                basis_domain.add_metas(self.data.domain.get_metas())
                 basis_matrix = Orange.data.Table(basis_domain, self.data)
 
         self.send("Learner", learner)
@@ -103,11 +113,13 @@ class OWEarth(OWWidget):
         self.send("Basis Matrix", basis_matrix)
 
     def sendReport(self):
-        self.reportSettings("Learning parameters", 
-                            [("Degree", self.degree),
-                             ("Terms", self.terms if self.terms >= 2 else "Automatic"),
-                             ("Knot penalty", "%.2f" % self.penalty)
-                             ])
+        self.reportSettings(
+            "Learning parameters",
+            [("Degree", self.degree),
+             ("Terms", self.terms if self.terms >= 2 else "Automatic"),
+             ("Knot penalty", "%.2f" % self.penalty)
+             ])
+
         self.reportData(self.data)
 
 if __name__ == "__main__":
