@@ -17,7 +17,6 @@ injection of add-ons into the namespace.
 
 #TODO Document this module.
 
-import socket
 import shelve
 import xmlrpclib
 import warnings
@@ -25,11 +24,17 @@ import re
 import pkg_resources
 import tempfile
 import tarfile
+import zipfile
 import shutil
 import os
 import sys
 import platform
 import subprocess
+import urllib2
+import urlparse
+import posixpath
+import site
+import itertools
 
 from collections import namedtuple, defaultdict
 from contextlib import closing
@@ -38,7 +43,6 @@ import Orange.utils.environ
 
 ADDONS_ENTRY_POINT="orange.addons"
 
-socket.setdefaulttimeout(120)  # In seconds.
 
 OrangeAddOn = namedtuple('OrangeAddOn', ['name', 'available_version', 'installed_version', 'summary', 'description',
                                          'author', 'docs_url', 'keywords', 'homepage', 'package_url',
@@ -49,6 +53,7 @@ OrangeAddOn = namedtuple('OrangeAddOn', ['name', 'available_version', 'installed
 INDEX_RE = "[^a-z0-9-']"  # RE for splitting entries in the search index
 
 AOLIST_FILE = os.path.join(Orange.utils.environ.orange_settings_dir, "addons.shelve")
+
 def open_addons():
     try:
         addons = shelve.open(AOLIST_FILE, 'c')
@@ -60,9 +65,10 @@ def open_addons():
         addons = shelve.open(AOLIST_FILE, 'n')
     return addons
 
-global addons_corrupted
-with closing(open_addons()) as addons:
-    addons_corrupted = len(addons)==0
+
+def addons_corrupted():
+    with closing(open_addons()) as addons:
+        return len(addons) == 0
 
 addon_refresh_callback = []
 
@@ -112,7 +118,6 @@ def refresh_available_addons(force=False, progress_callback=None):
     except:
         readthedocs = None
 
-    global addons_corrupted
     docs = {}
     if progress_callback:
         progress_callback(len(pkg_dict)+1, 1)
@@ -147,7 +152,6 @@ def refresh_available_addons(force=False, progress_callback=None):
                     warnings.warn('Could not load data for the following add-on: %s'%name)
             if progress_callback:
                 progress_callback(len(pkg_dict)+1, i+2)
-        addons_corrupted = False
 
     rebuild_index()
 
@@ -179,6 +183,117 @@ def load_installed_addons():
     rebuild_index()
 
 
+def open_archive(path, mode="r"):
+    """
+    Return an open archive file object (zipfile.ZipFile or tarfile.TarFile).
+    """
+    _, ext = os.path.splitext(path)
+    if ext == ".zip":
+        # TODO: should it also open .egg, ...
+        archive = zipfile.ZipFile(path, mode)
+
+    elif ext in (".tar", ".gz", ".bz2", ".tgz", ".tbz2", ".tb2"):
+        archive = tarfile.open(path, mode)
+
+    return archive
+
+
+member_info = namedtuple(
+    "member_info",
+    ["info",  # original info object (Tar/ZipInfo)
+     "path",  # filename inside the archive
+     "linkname",  # linkname if applicable
+     "issym",  # True if sym link
+     "islnk",  # True if hardlink
+     ]
+)
+
+
+def archive_members(archive):
+    """
+    Given an open archive return an iterator of `member_info` instances.
+    """
+    if isinstance(archive, zipfile.ZipFile):
+        def converter(info):
+            return member_info(info, info.filename, None, False, False)
+
+        return itertools.imap(converter, archive.infolist())
+    elif isinstance(archive, tarfile.TarFile):
+        def converter(info):
+            return member_info(info, info.name, info.linkname,
+                               info.issym(), info.islnk())
+        return itertools.imap(converter, archive.getmembers())
+    else:
+        raise TypeError
+
+
+def resolve_path(path):
+    """
+    Return a normalized real path.
+    """
+    return os.path.normpath(os.path.realpath(os.path.abspath(path)))
+
+
+def is_badfile(member, base_dir):
+    """
+    Would extracting `member_info` instance write outside of `base_dir`.
+    """
+    path = member.path
+    full_path = resolve_path(os.path.join(base_dir, path))
+    return not full_path.startswith(base_dir)
+
+
+def is_badlink(member, base_dir):
+    """
+    Would extracting `member_info` instance create a link to outside
+    of `base_dir`.
+
+    """
+    if member.issym or member.islnk:
+        dirname = os.path.dirname(member.path)
+        full_path = resolve_path(os.path.join(dirname, member.linkname))
+        return not full_path.startswith(base_dir)
+    else:
+        return False
+
+
+def check_safe(member, base_dir):
+    """
+    Check if member is safe to extract to base_dir or raise an exception.
+    """
+    path = member.path
+    drive, path = os.path.splitdrive(path)
+
+    if drive != "":
+        raise ValueError("Absolute path in archive")
+
+    if path.startswith("/"):
+        raise ValueError("Absolute path in archive")
+
+    base_dir = resolve_path(base_dir)
+
+    if is_badfile(member, base_dir):
+        raise ValueError("Extract outside %r" % base_dir)
+    if is_badlink(member, base_dir):
+        raise ValueError("Link outside %r" % base_dir)
+
+    return True
+
+
+def extract_archive(archive, path="."):
+    """
+    Extract the contents of `archive` to `path`.
+    """
+    if isinstance(archive, basestring):
+        archive = open_archive(archive)
+
+    members = archive_members(archive)
+
+    for member in members:
+        if check_safe(member, path):
+            archive.extract(member.info, path)
+
+
 def run_setup(setup_script, args):
     """
     Run `setup_script` with `args` in a subprocess, using
@@ -194,7 +309,12 @@ def run_setup(setup_script, args):
         # by default a new console window would show up when executing the
         # script
         startupinfo = subprocess.STARTUPINFO()
-        startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
+        if hasattr(subprocess, "STARTF_USESHOWWINDOW"):
+            startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
+        else:
+            # This flag was missing in inital releases of 2.7
+            startupinfo.dwFlags |= subprocess._subprocess.STARTF_USESHOWWINDOW
+
         extra_kwargs["startupinfo"] = startupinfo
 
     subprocess.check_output([executable, setup_script] + args,
@@ -206,27 +326,34 @@ def run_setup(setup_script, args):
 def install(name, progress_callback=None):
     if progress_callback:
         progress_callback(1, 0)
-    import site
-    try:
-        import urllib
-        rh = (lambda done, bs, fs: progress_callback(fs/bs, done)) if progress_callback else None
-        with closing(open_addons()) as addons:
-            egg = urllib.urlretrieve(addons[name.lower()].release_url, reporthook=rh)[0]
-    except Exception, e:
-        raise Exception("Unable to download add-on from repository: %s" % e)
+
+    with closing(open_addons()) as addons:
+        addon = addons[name.lower()]
+    release_url = addon.release_url
 
     try:
-        try:
-            tmpdir = tempfile.mkdtemp()
-            egg_contents = tarfile.open(egg)
-            egg_contents.extractall(tmpdir)
-            with closing(open_addons()) as addons:
-                setup_py = os.path.join(tmpdir, name+'-'+addons[name.lower()].available_version, 'setup.py')
-        except Exception, e:
-            raise Exception("Unable to unpack add-on: %s" % e)
+        tmpdir = tempfile.mkdtemp()
+
+        stream = urllib2.urlopen(release_url, timeout=120)
+
+        parsed_url = urlparse.urlparse(release_url)
+        package_name = posixpath.basename(parsed_url.path)
+        package_path = os.path.join(tmpdir, package_name)
+
+        progress_cb = (lambda value: progress_callback(value, 0)) \
+                      if progress_callback else None
+        with open(package_path, "wb") as package_file:
+            Orange.utils.copyfileobj(
+                stream, package_file, progress=progress_cb)
+
+        extract_archive(package_path, tmpdir)
+
+        setup_py = os.path.join(tmpdir, name + '-' + addon.available_version,
+                                'setup.py')
 
         if not os.path.isfile(setup_py):
-            raise Exception("Unable to install add-on - it is not properly packed.")
+            raise Exception("Unable to install add-on - it is not properly "
+                            "packed.")
 
         switches = []
         if site.USER_SITE in sys.path:   # we're not in a virtualenv
