@@ -122,7 +122,10 @@ bool TFilter_Python::operator()(const TExample &ex)
 
 PFilter TFilter_Python::deepCopy() const
 {
-    PyObject *result = PyObject_CallMethod((PyObject *) myWrapper, "deepCopy",
+    const char * method = PyObject_HasAttrString((PyObject *)myWrapper, "deep_copy") ? "deep_copy" : "deepCopy";
+
+    PyObject *result = PyObject_CallMethod((PyObject *) myWrapper,
+            (char *) method,
             NULL);
     if (!result)
         raiseError("An exception has been thrown in method deepCopy!");
@@ -236,8 +239,10 @@ void TMeasureAttribute_Python::thresholdFunction(TFloatFloatList &res,
 
     PyObject *args = Py_BuildValue("NNNi", WrapOrange(var), WrapOrange(gen),
             WrapOrange(apriorClass), weightID);
-    PyObject *pyres = callMethod("thresholdFunction", (PyObject *) myWrapper,
-            args);
+    PyObject *pyres = callMethod(
+        PyObject_HasAttrString((PyObject *)myWrapper, "threshold_function") ? "threshold_function" : "thresholdFunction",
+        (PyObject *) myWrapper,
+        args);
     Py_DECREF(args);
 
     if (!PyList_Check(pyres)) {
