@@ -339,6 +339,8 @@ TExamplesDistanceConstructor_Euclidean::TExamplesDistanceConstructor_Euclidean()
 TExamplesDistanceConstructor_Manhattan::TExamplesDistanceConstructor_Manhattan()
 {}
 
+TExamplesDistanceConstructor_Lp::TExamplesDistanceConstructor_Lp(): p(1.0)
+{}
 
 PExamplesDistance TExamplesDistanceConstructor_Maximal::operator()(PExampleGenerator egen, const int &weightID, PDomainDistributions ddist, PDomainBasicAttrStat bstat) const
 { return mlnew TExamplesDistance_Maximal(ignoreClass, normalize, ignoreUnknowns, egen, weightID, ddist, bstat); }
@@ -351,7 +353,8 @@ PExamplesDistance TExamplesDistanceConstructor_Manhattan::operator()(PExampleGen
 PExamplesDistance TExamplesDistanceConstructor_Euclidean::operator()(PExampleGenerator egen, const int &weightID, PDomainDistributions ddist, PDomainBasicAttrStat bstat) const
 { return mlnew TExamplesDistance_Euclidean(ignoreClass, normalize, ignoreUnknowns, egen, weightID, ddist, bstat); }
 
-
+PExamplesDistance TExamplesDistanceConstructor_Lp::operator()(PExampleGenerator egen, const int &weightID, PDomainDistributions ddist, PDomainBasicAttrStat bstat) const
+{ return mlnew TExamplesDistance_Lp(ignoreClass, normalize, ignoreUnknowns, egen, weightID, ddist, bstat, p); }
 
 TExamplesDistance_Maximal::TExamplesDistance_Maximal()
 {}
@@ -402,7 +405,18 @@ TExamplesDistance_Euclidean::TExamplesDistance_Euclidean(const bool &ignoreClass
   }
 }
 
+TExamplesDistance_Lp::TExamplesDistance_Lp()
+{
+	this->p = 1.0;
+}
+TExamplesDistance_Lp::TExamplesDistance_Lp(float p)
+{
+	this->p = p;
+}
 
+TExamplesDistance_Lp::TExamplesDistance_Lp(const bool &ignoreClass, const bool &normalize, const bool &ignoreUnknowns, PExampleGenerator egen, const int &weightID, PDomainDistributions ddist, PDomainBasicAttrStat dstat, float _p)
+: TExamplesDistance_Normalized(ignoreClass, normalize, ignoreUnknowns, egen, weightID, ddist, dstat), p(_p)
+{}
 
 float TExamplesDistance_Maximal::operator ()(const TExample &e1, const TExample &e2) const 
 { 
@@ -479,6 +493,15 @@ float TExamplesDistance_Euclidean::operator ()(const TExample &e1, const TExampl
 }
 
 
+float TExamplesDistance_Lp::operator ()(const TExample &e1, const TExample &e2) const
+{
+  vector<float> difs;
+  getDifs(e1, e2, difs);
+  float dist = 0.0;
+  const_ITERATE(vector<float>, di, difs)
+    dist += pow(fabs(*di), p);
+  return pow(dist, 1.0f / p);
+}
 
 
 TExamplesDistanceConstructor_Relief::TExamplesDistanceConstructor_Relief()
@@ -491,7 +514,7 @@ PExamplesDistance TExamplesDistanceConstructor_Relief::operator()(PExampleGenera
 
   PVariable otherAttribute = domain.hasOtherAttributes();
   if (otherAttribute)
-    raiseError("domain has attributes whose type is not supported by ReliefF (e.g. '%s')", otherAttribute->name.c_str());
+    raiseError("domain has attributes whose type is not supported by ReliefF (e.g. '%s')", otherAttribute->get_name().c_str());
 
   // for continuous attributes BasicAttrStat suffices; for discrete it does not
   const bool hasDiscrete = domain.hasDiscreteAttributes() || domain.classVar && (domain.classVar->varType == TValue::INTVAR);
@@ -518,7 +541,7 @@ PExamplesDistance TExamplesDistanceConstructor_Relief::operator()(PExampleGenera
   if (ddist)
     edr->distributions->normalize();
   
-  for(int attrIndex = 0, nAttrs = gen->domain->attributes->size(); attrIndex != nAttrs; attrIndex++)
+  for(int attrIndex = 0, nAttrs = gen->domain->variables->size(); attrIndex != nAttrs; attrIndex++)
     if (domain.variables->at(attrIndex)->varType == TValue::FLOATVAR) {
       if (bstat) {
         const TBasicAttrStat &bas = bstat->at(attrIndex).getReference();
@@ -559,7 +582,7 @@ float TExamplesDistance_Relief::operator()(const TExample &e1, const TExample &e
 
   const bool hasDistributions = bool(distributions);
 
-  TExample::const_iterator e1i(e1.begin());
+  TExample::const_iterator e1i(e1.begin()), e1end((e1.domain->classVar)? e1.end() - 1: e1.end());
   TExample::const_iterator e2i(e2.begin());
   TFloatList::const_iterator avgi(averages->begin()), avge(averages->end()),
                              nori(normalizations->begin()),
@@ -570,7 +593,7 @@ float TExamplesDistance_Relief::operator()(const TExample &e1, const TExample &e
     di = distributions->begin();
 
   float dist = 0.0;
-  for(; avgi!=avge; e1i++, e2i++, avgi++, nori++, btsi++) {
+  for(; e1i!=e1end; e1i++, e2i++, avgi++, nori++, btsi++) {
     float dd = 0.0;
     const TValue &v1 = *e1i, &v2 = *e2i;
     if (v1.varType==TValue::INTVAR) {             // discrete
