@@ -66,8 +66,9 @@ public:
   __REGISTER_CLASS
 
   PVariable classVar; //PR class variable
-  PVarList attributes; //PR list of attributes, excluding the class
+  PVarList attributes; //PR(+features) list of attributes, excluding the class
   PVarList variables; //PR list of attributes, including the class at the end of the list
+  PVarList classVars; //PR list of class variables, if multiple; else None
 
   TMetaVector metas;
 
@@ -148,7 +149,7 @@ public:
   PVariable hasContinuousAttributes(bool checkClass = true) const;
   PVariable hasOtherAttributes(bool checkClass = true) const;
   
-  void addToCRC(unsigned long &crc) const;
+  void addToCRC(unsigned int &crc) const;
   int sumValues() const;
 };
 
