@@ -39,7 +39,7 @@ public:
   __REGISTER_CLASS
 
   PDomain domain; //PR Example's domain
-  TValue *values, *values_end;
+  TValue *values, *values_end, *classes_end;
   TMetaValues meta;
   string *name;
   int id; //P
@@ -155,12 +155,12 @@ public:
   int  compare(const TExample &other, const bool ignoreClass = false) const;
   bool compatible(const TExample &other, const bool ignoreClass = false) const;
 
-  void addToCRC(unsigned long &crc, const bool includeMetas = false) const;
+  void addToCRC(unsigned int &crc, const bool includeMetas = false) const;
   int sumValues(const bool includeMetas = false) const;
   
   bool hasMissing() const
   {
-    for(TValue const *vi = values, *ve = values_end; vi!=ve; vi++)
+    for(TValue const *vi = values, *ve = classes_end; vi!=ve; vi++)
       if (vi->isSpecial())
         return true;
     return false;
