@@ -39,14 +39,15 @@ gsl_error_handler_t *fooerrh = gsl_set_error_handler(my_gsl_error_handler);
 */
 
 void parseMatrixContents(PExampleGenerator egen, const int &weightID, const char *contents, const int &multiTreatment,
-                         bool &hasClass, bool &classVector, bool &weightVector, bool &classIsDiscrete, int &columns,
+                         bool &hasClass, bool &classVector, bool &multiclassVector, bool &weightVector, bool &classIsDiscrete, int &columns,
                          vector<bool> &include)
 {
   hasClass = bool(egen->domain->classVar);
 
   columns = 0;
-  int classIncluded = 0, attrsIncluded = 0, weightIncluded = 0;
+  int classIncluded = 0, attrsIncluded = 0, weightIncluded = 0, multiclassIncluded = 0;
   bool attrsRequested = false, classRequested = false, weightRequested = false;
+  bool multiclassRequested = false;
   const char *cp;
   for(cp = contents; *cp && (*cp!='/'); cp++) {
     switch (*cp) {
@@ -62,6 +63,10 @@ void parseMatrixContents(PExampleGenerator egen, const int &weightID, const char
       case 'w': weightIncluded++;
                 break;
 
+      case 'M': multiclassRequested = true;
+      case 'm': multiclassIncluded++;
+                break;
+
       case '0':
       case '1': columns++;
                 break;
@@ -71,6 +76,7 @@ void parseMatrixContents(PExampleGenerator egen, const int &weightID, const char
   }
 
   classVector = false;
+  multiclassVector = false;
   weightVector = false;
 
   if (*cp)
@@ -87,6 +93,9 @@ void parseMatrixContents(PExampleGenerator egen, const int &weightID, const char
 
         case 'w': weightVector = (weightID != 0); break;
         case 'W': weightVector = true; break;
+
+        case 'm': multiclassVector = (egen->domain->classVars->size() != 0); break;
+        case 'M': multiclassVector = true; break;
         default:
           raiseErrorWho("parseMatrixContents", "unrecognized character '%c' in format string '%s')", *cp, contents);
       }
@@ -117,6 +126,10 @@ void parseMatrixContents(PExampleGenerator egen, const int &weightID, const char
       columns += weightIncluded;
   }
 
+  if (multiclassIncluded || multiclassVector) {
+      columns += multiclassIncluded * egen->domain->classVars->size();
+  }
+
   include.clear();
 
   if (attrsIncluded) {
@@ -144,13 +157,13 @@ void parseMatrixContents(PExampleGenerator egen, const int &weightID, const char
               break;
 
             default:
-              raiseErrorWho("parseMatrixContents", "attribute '%s' is multinomial", (*vi)->name.c_str());
+              raiseErrorWho("parseMatrixContents", "attribute '%s' is multinomial", (*vi)->get_name().c_str());
           }
       }
       else {
         attrs_in++;
         include.push_back(true);
-        raiseWarning(PyExc_OrangeKernelWarning, "attribute '%s' is of unsupported type", (*vi)->name.c_str());
+        raiseWarning(PyExc_OrangeKernelWarning, "attribute '%s' is of unsupported type", (*vi)->get_name().c_str());
       }
     }
 
