@@ -51,17 +51,17 @@
 #include "discretize.hpp"
 
 
-ABSTRACT(Discretizer, TransformValue)
-C_NAMED(EquiDistDiscretizer, Discretizer, "([numberOfIntervals=, firstCut=, step=])")
-C_NAMED(IntervalDiscretizer, Discretizer, "([points=])")
-C_NAMED(ThresholdDiscretizer, Discretizer, "([threshold=])")
-C_NAMED(BiModalDiscretizer, Discretizer, "([low=, high=])")
+ABSTRACT(Discretizer - Orange.feature.discretization.Discretizer, TransformValue)
+C_NAMED(EquiDistDiscretizer - Orange.feature.discretization.EqualWidthDiscretizer, Discretizer, "([numberOfIntervals=, firstCut=, step=])")
+C_NAMED(IntervalDiscretizer - Orange.feature.discretization.IntervalDiscretizer, Discretizer, "([points=])")
+C_NAMED(ThresholdDiscretizer - Orange.feature.discretization.ThresholdDiscretizer, Discretizer, "([threshold=])")
+C_NAMED(BiModalDiscretizer - Orange.feature.discretization.BiModalDiscretizer, Discretizer, "([low=, high=])")
 
-ABSTRACT(Discretization, Orange)
-C_CALL (EquiDistDiscretization, Discretization, "() | (attribute, examples[, weight, numberOfIntervals=]) -/-> Variable")
-C_CALL (   EquiNDiscretization, Discretization, "() | (attribute, examples[, weight, numberOfIntervals=]) -/-> Variable")
-C_CALL ( EntropyDiscretization, Discretization, "() | (attribute, examples[, weight]) -/-> Variable")
-C_CALL ( BiModalDiscretization, Discretization, "() | (attribute, examples[, weight]) -/-> Variable")
+ABSTRACT(Discretization - Orange.feature.discretization.Discretization, Orange)
+C_CALL (EquiDistDiscretization - Orange.feature.discretization.EqualWidth, Discretization, "() | (attribute, examples[, weight, numberOfIntervals=]) -/-> Variable")
+C_CALL (   EquiNDiscretization - Orange.feature.discretization.EqualFreq, Discretization, "() | (attribute, examples[, weight, numberOfIntervals=]) -/-> Variable")
+C_CALL ( EntropyDiscretization - Orange.feature.discretization.Entropy, Discretization, "() | (attribute, examples[, weight]) -/-> Variable")
+C_CALL ( BiModalDiscretization - Orange.feature.discretization.BiModal, Discretization, "() | (attribute, examples[, weight]) -/-> Variable")
 
 
 PyObject *Discretization_call(PyObject *self, PyObject *args, PyObject *keywords) PYDOC("(attribute, examples[, weight]) -> Variable")
@@ -115,11 +115,11 @@ PyObject *EquiDistDiscretizer_get_points(PyObject *self)
 #include "transval.hpp"
 
 C_NAMED(MapIntValue, TransformValue, "([mapping=])")
-C_NAMED(Discrete2Continuous, TransformValue, "([value=])")
+C_NAMED(Discrete2Continuous - Orange.feature.discretization.Discrete2Continuous, TransformValue, "([value=])")
 C_NAMED(Ordinal2Continuous, TransformValue, "([nvalues=])")
 C_NAMED(NormalizeContinuous, TransformValue, "([average=, span=])")
 
-C_NAMED(DomainContinuizer, Orange, "(domain|examples, convertClass=, invertClass=, zeroBased=, normalizeContinuous=, baseValueSelection=) -/-> Domain")
+C_NAMED(DomainContinuizer - Orange.data.continuization.DomainContinuizer, Orange, "(domain|examples, convertClass=, invertClass=, zeroBased=, normalizeContinuous=, baseValueSelection=) -/-> Domain")
 
 int getTargetClass(PVariable classVar, PyObject *pyval)
 {
@@ -329,6 +329,8 @@ PyObject *Preprocessor_selectionVector(PyObject *self, PyObject *args, PyObject
 
 typedef MapMethods<PVariableFilterMap, TVariableFilterMap, PVariable, PValueFilter> TMM_VariableFilterMap;
 
+INITIALIZE_MAPMETHODS(TMM_VariableFilterMap, &PyOrVariable_Type, &PyOrValueFilter_Type, _orangeValueFromPython<PVariable>, _orangeValueFromPython<PValueFilter>, _orangeValueToPython<PVariable>, _orangeValueToPython<PValueFilter>)
+
 int VariableFilterMap_setitemlow(TVariableFilterMap *aMap, PVariable var, PyObject *pyvalue)
 {
   PValueFilter value;
@@ -489,7 +491,6 @@ PyObject *kaplanMeier(PyObject *, PyObject *args) PYARGS(METH_VARARGS, "(example
 
 
 // modified setitem to accept intervals/names of values
-INITIALIZE_MAPMETHODS(TMM_VariableFilterMap, &PyOrVariable_Type, &PyOrValueFilter_Type, _orangeValueFromPython<PVariable>, _orangeValueFromPython<PValueFilter>, _orangeValueToPython<PVariable>, _orangeValueToPython<PValueFilter>)
 
 PVariableFilterMap PVariableFilterMap_FromArguments(PyObject *arg) { return TMM_VariableFilterMap::P_FromArguments(arg); }
 PyObject *VariableFilterMap_FromArguments(PyTypeObject *type, PyObject *arg) { return TMM_VariableFilterMap::_FromArguments(type, arg); }
