@@ -32,6 +32,19 @@
 #include "table.ppp"
 
 
+TExampleTable::TExampleTable(const TExampleTable& orig)
+: TExampleGenerator(orig.domain),
+  examples(NULL),
+  _Last(NULL),
+  _EndSpace(NULL),
+  ownsExamples(true)
+{
+  reserve(orig.size());
+  for(int i = 0; i < orig.size(); ++i){
+	  push_back(new TExample(orig.at(i)));
+  }
+}
+
 TExampleTable::TExampleTable(PDomain dom, bool owns)
 : TExampleGenerator(dom),
   examples(NULL),
@@ -118,7 +131,7 @@ TExampleTable::TExampleTable(PExampleGeneratorList tables)
       bool notfirst = 0;
       for(vector<pair<int, int> >::const_iterator sdmmi((*dmmi).begin()), sdmme((*dmmi).end()); sdmmi!=sdmme; sdmmi++) {
         if (!mergeTwoValues(*ei, (*iterators[(*sdmmi).first])[(*sdmmi).second], notfirst++ != 0))
-          raiseError("mismatching value of attribute '%s' in example #%i", (*vi)->name.c_str(), exno);
+          raiseError("mismatching value of attribute '%s' in example #%i", (*vi)->get_name().c_str(), exno);
       }
     }
 
@@ -128,8 +141,8 @@ TExampleTable::TExampleTable(PExampleGeneratorList tables)
         if (example->hasMeta((*mvi).first)) {
           if (!mergeTwoValues(example->getMeta((*mvi).first), (*mvi).second, true)) {
             PVariable metavar = domain->getMetaVar((*mvi).first, false);
-            if (metavar && metavar->name.length())
-              raiseError("Meta attribute '%s' has ambiguous values on example #%i", metavar->name.c_str(), exno);
+            if (metavar && metavar->get_name().length())
+              raiseError("Meta attribute '%s' has ambiguous values on example #%i", metavar->get_name().c_str(), exno);
             else
               raiseError("Meta attribute %i has ambiguous values on example #%i", (*mvi).first, exno);
           }
@@ -682,6 +695,80 @@ void TExampleTable::changeDomain(PDomain dom, bool filterMetas)
 }
 
 
+void TExampleTable::pickClass(PVariable classVar)
+{
+    if (!ownsExamples) {
+        raiseError("Cannot change the class in a table of references");
+    }
+    const int attrs = domain->attributes->size();
+    const int classes = domain->classVars->size();
+    TDomain *newDomain;
+    int classPos;
+    int end_off = 0;
+    if (!classVar) {
+        if (!classes) {
+            return;
+        }
+        end_off = -1;
+        newDomain = mlnew TDomain(classVar, domain->attributes.getReference()); // No class
+        newDomain->classVars->push_back(domain->classVar);
+        PITERATE(TVarList, ci, domain->classVars) {
+            if (*ci != classVar) {
+                newDomain->classVars->push_back(*ci);
+            }
+        }
+        classPos = attrs;
+    }
+    else {
+        classPos = attrs;
+        if (domain->classVar) {
+            classPos += 1;
+        }
+        else {
+            end_off += 1;
+        }
+        TVarList::const_iterator mci(domain->classVars->begin()), mce(domain->classVars->end());
+        for(; (mci != mce) && (*mci != classVar); ++mci, ++classPos);
+        if (mci == mce) {
+            raiseError("Domain has no class %s", classVar->get_name().c_str());
+        }
+        newDomain = mlnew TDomain(classVar, domain->attributes.getReference());
+        newDomain->classVars = mlnew TVarList();
+        TVarList::iterator ci(domain->classVars->begin());
+        TVarList::const_iterator const ce(domain->classVars->end());
+        if (!domain->classVar && (classPos != attrs)) {
+            ci++;
+        }
+        for(; ci != ce; ci++) {
+            if (*ci == classVar) {
+                if (domain->classVar) {
+                    newDomain->classVars->push_back(domain->classVar);
+                }
+                else if (classPos != attrs) {
+                    newDomain->classVars->push_back(domain->classVars->front());
+                }
+            }
+            else {
+                newDomain->classVars->push_back(*ci);
+            }
+        }
+    }
+    ITERATE(TMetaVector, mi, domain->metas) {
+        newDomain->metas.push_back(*mi);
+    }
+    PDomain wdomain = newDomain;
+    domain = wdomain;
+    for(TExample **ri = examples; ri != _Last; ri++) {
+        (**ri).domain = wdomain;
+        if (classPos != attrs) {
+            TValue t = (**ri).values[attrs];
+            (**ri).values[attrs] = (**ri).values[classPos];
+            (**ri).values[classPos] = t;
+        }
+        (**ri).values_end += end_off;
+    }
+}
+
 void TExampleTable::addMetaAttribute(const int &id, const TValue &value)
 { 
   PEITERATE(ei, this)
@@ -752,7 +839,7 @@ void TExampleTable::sort(vector<int> &sortOrder)
           const int intV = val.isSpecial() ? noVal : val.intV;
           if (intV > noVal) {
             free(temp);
-            raiseError("value out attribute '%s' of range", domain->variables->operator[](*bi)->name.c_str());
+            raiseError("value out attribute '%s' of range", domain->variables->operator[](*bi)->get_name().c_str());
           }
           valf[intV]++;
         }
@@ -808,7 +895,7 @@ void TExampleTable::shuffle()
 }
 
 int TExampleTable::checkSum(const bool includeMetas)
-{ unsigned long crc;
+{ unsigned int crc;
   INIT_CRC(crc);
 
   for(TExample **ei = examples, **ee = _Last; ei!=ee; (*ei++)->addToCRC(crc, includeMetas));
@@ -818,7 +905,7 @@ int TExampleTable::checkSum(const bool includeMetas)
 }
 
 int TExampleTable::checkSum(const bool includeMetas) const
-{ unsigned long crc;
+{ unsigned int crc;
   INIT_CRC(crc);
 
   for(TExample **ei = examples, **ee = _Last; ei!=ee; (*ei++)->addToCRC(crc, includeMetas));
