@@ -35,12 +35,14 @@ public:
   TExample **examples;
   TExample **_Last, **_EndSpace;
   PRandomGenerator randomGenerator; //P random generator used by randomExample
-  PExampleGenerator lock; //PR the real owner of examples
-  bool ownsExamples; //PR if false, examples in this table are references to examples in another table
+  PExampleGenerator lock; //PR (+owner) the real owner of examples
+  bool ownsExamples; //PR (+owns_instances) if false, examples in this table are references to examples in another table
 
   // Iterates through examples of basevector
   #define baseITERATE(x) ITERATE(vector<TExample>, x, examples)
 
+  TExampleTable(const TExampleTable& orig);
+
   TExampleTable(PDomain, bool owns = true);
   TExampleTable(PExampleGenerator orig, bool owns = true); // also copies examples
   TExampleTable(PDomain, PExampleGenerator orig, bool filterMetas = false); // owns = true (cannot change domain of references); copies examples
@@ -83,6 +85,7 @@ public:
   bool randomExample(TExample &);
 
   virtual void changeDomain(PDomain, bool filterMetas = false);
+  virtual void pickClass(PVariable);
 
 protected:
   virtual void increaseIterator(TExampleIterator &);
