@@ -38,7 +38,6 @@ TNetwork::TNetwork(TNetwork *net)
 			double *w = getOrCreateEdge(v1, *ni);
 			*w = *net->getEdge(v1, *ni);
 		}
-
 		vertices.push_back(v1);
 		optimize.insert(v1);
 	}
@@ -461,6 +460,7 @@ TNetworkHierarchyNode *TNetworkHierarchy::getNodeByVertex(int vertex)
 #include "externs.px"
 #include "orange_api.hpp"
 WRAPPER(GraphAsList);
+
 PyObject *Network_new(PyTypeObject *type, PyObject *args, PyObject *kwds) BASED_ON (GraphAsList, "(nVertices, directed[, nEdgeTypes])")
 {
 	PyTRY
@@ -488,7 +488,7 @@ PyObject *Network_new(PyTypeObject *type, PyObject *args, PyObject *kwds) BASED_
 		else if (PyOrGraphAsList_Check(pygraph))
 		{
 			//cout << "2" << endl;
-   		TGraphAsList *graph = PyOrange_AsGraphAsList(pygraph).getUnwrappedPtr();
+			TGraphAsList *graph = dynamic_cast<TGraphAsList *>(PyOrange_AsOrange(pygraph).getUnwrappedPtr());
 			TNetwork *network = mlnew TNetwork(graph);
 			
 			// set graphs attribut items of type ExampleTable to subgraph
@@ -1772,7 +1772,6 @@ PyObject *Network_parseNetwork(PyObject *self, PyObject *args) PYARGS(METH_VARAR
 								if (!directed) {
 									*(graph->getOrCreateEdge(i2 - 1, i1 - 1) + i) = i3;
 								}
-								
 							}
 						}
 					}
@@ -1790,4 +1789,4 @@ PyObject *Network_parseNetwork(PyObject *self, PyObject *args) PYARGS(METH_VARAR
   PyCATCH
 }
 
-#include "network.px"
\ No newline at end of file
+#include "network.px"
